@@ -4,7 +4,7 @@
 //!
 //! # Introduction
 //!
-//! A proportional–integral–derivative controller is a control loop mechanism
+//! A proportional-integral-derivative controller is a control loop mechanism
 //! employing feedback that is widely used in industrial control systems
 //! and a variety of other applications requiring continuously modulated control.
 //!
@@ -50,13 +50,14 @@ pub struct PID<F, const W: usize> {
     pub ki: F,
     /// Derivative gain.
     pub kd: F,
-    ///Feed forward gain
+    /// Feed forward gain
     pub kf: F,
-    ///Velocity
+    /// Velocity
     pub kv: F,
     last_sp: F,
     last_error_idx: usize,
     errors: [F; W],
+    err_history: F,
 }
 
 impl<F, const W: usize> PID<F, W>
@@ -89,7 +90,8 @@ where
             kv: kv.into(),
             last_sp: F::default(),
             errors: [F::default(); W],
-            last_error_idx: 0,
+            last_error_idx: usize::default(),
+            err_history: F::default(),
         }
     }
 
@@ -109,7 +111,7 @@ where
     /// ```
     pub fn reset(&mut self) {
         self.last_sp = F::default();
-        self.last_error_idx = 0;
+        self.last_error_idx = usize::default();
         self.errors = [F::default(); W];
     }
 
@@ -131,22 +133,28 @@ where
         let error = sp - fb;
 
         let error_delta = error - self.errors[self.last_error_idx];
-        self.last_error_idx += 1;
-        if self.last_error_idx >= W {
-            self.last_error_idx = 0
-        }
-        self.errors[self.last_error_idx] = error;
-        let err_history = self.errors.iter().fold(F::default(), |acc, i| acc + *i);
+        self.push_error(error);
 
         let sp_delta = sp - self.last_sp;
         self.last_sp = sp;
 
         let p = self.kp * error;
-        let i = self.ki * err_history;
+        let i = self.ki * self.err_history;
         let d = self.kd * error_delta;
         let f = self.kf * sp_delta;
         let v = self.kv * fb;
 
         p + i + d + f + v
     }
+
+    fn push_error(&mut self, error: F) {
+        self.last_error_idx += 1;
+        if self.last_error_idx >= W {
+            self.last_error_idx = 0;
+        }
+
+        self.err_history = self.err_history - self.errors[self.last_error_idx];
+        self.err_history = self.err_history + error;
+        self.errors[self.last_error_idx] = error;
+    }
 }
